local LrPathUtils = import("LrPathUtils")
local LrTasks = import("LrTasks")
local LrFileUtils = import("LrFileUtils")
local LrView = import("LrView")
local LrStringUtils = import("LrStringUtils")
local LrDate = import("LrDate")
local LrDialogs = import("LrDialogs")
local LrApplication = import("LrApplication")

local ExportProcessRenderedPhotos = {}

ExportProcessRenderedPhotos.exportPresetFields = {
	{ key = 'hide',		default = true },
	{ key = 'presetGroup',	default = 1 },
	{ key = 'preset', default = 1 },
	{ key = 'alignment', default = false },
	{ key = 'ghostsReduction', default = false },
	{ key = 'chromaticAberrationReduction', default = false },
    { key = 'exportImageFormat', default = 'ORIGINAL' },
}

ExportProcessRenderedPhotos.hideSections = {'exportLocation', 'fileNaming', 'imageSettings', 'outputSharpening', 'metadata', 'watermarking', 'video'}
ExportProcessRenderedPhotos.canExportToTemporaryLocation = true
ExportProcessRenderedPhotos.allowFileFormats = {'ORIGINAL', 'TIFF'}

local random = math.random
local function uuid()
    local template ='xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
    return string.gsub(template, '[xy]', function (c)
        local v = (c == 'x') and random(0, 0xf) or random(8, 0xb)
        return string.format('%x', v)
    end)
end

local validPresets = nil
local validGroups = nil

function ExportProcessRenderedPhotos.processRenderedPhotos( functionContext, exportContext )

	local presets = validPresets
	
	local groupPresets = nil

	local selectedGroup = exportContext.propertyTable.presetGroup
	
	if selectedGroup >= 2 then
		groupPresets = presets[exportContext.propertyTable.presetGroup - 1]
	end
	
	
	local selectedPreset = nil
	
	if groupPresets ~= nil then
		selectedPreset = groupPresets[exportContext.propertyTable.preset]
	end
	
	local exportSession = exportContext.exportSession

	-- Make a local reference to the export parameters.
	
	local exportParams = exportContext.propertyTable
	
	-- Set progress title.

	local nPhotos = exportSession:countRenditions()

	local progressScope = exportContext:configureProgress{
						title = nPhotos > 1
									and LOC( "$$$/MPExtrasPluginAppName/Upload/Progress=Uploading ^1 photos to MPExtrasAppName", nPhotos )
									or LOC ("$$$/MPExtrasPluginAppName/Upload/Progress/One=Uploading one photo to MPExtrasAppName"),
					}

	-- Save off uploaded photo IDs so we can take user to those photos later.
	
	local uploadedPhotoIds = {}
	
	local command = nil
	local bracket = nil
	local firstPhotoPath = nil
	local numberOfRenderedPhotos = 0	
	local collections = nil
	local keywords = nil

	-- metadata
	local copyName = nil
	local title = nil
	local caption = nil
	local copyright = nil
	local copyrightStatus = nil
	local creator = nil
	local location = nil
	local rating = nil
	local label = nil

	for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
	
		-- Get next photo.

		local success, pathOrMessage = rendition:waitForRender()
		
		-- Check for cancellation again after photo has been rendered.
		
		if progressScope:isCanceled() then 
			command = nil
			break 
		end
		

		if success then
			local photo = rendition.photo
			if collections == nil then
				collections = photo:getContainedCollections()
			else 
				collections = intersectArrays(collections, photo:getContainedCollections())
			end
			
			if keywords == nil then
				keywords = photo:getRawMetadata("keywords")
			else
				keywords = intersectArrays(keywords, photo:getRawMetadata("keywords"))
			end


			-- storing metadata

			if copyName == nil then
				copyName = photo:getFormattedMetadata("copyName")
			end

			if title == nil then
				title = photo:getFormattedMetadata("title")
			end

			if caption == nil then
				caption = photo:getFormattedMetadata("caption")
			end

			if copyright == nil then
				copyright = photo:getFormattedMetadata("copyright")
			end

			if copyrightStatus == nil then
				copyrightStatus = photo:getRawMetadata("copyrightState")
			end

			if creator == nil then
				creator = photo:getFormattedMetadata("creator")
			end

			if location == nil then
				location = photo:getFormattedMetadata("location")
			end

			if rating == nil then
				rating = photo:getRawMetadata("rating")
			end

			if label == nil then
				label = photo:getFormattedMetadata("label")
			end

			-- storing metadata end


			numberOfRenderedPhotos = numberOfRenderedPhotos + 1
			if command == nil then
				command = "MPLightroomBridgePath"
			end
			
			if firstPhotoPath == nil then
				firstPhotoPath = pathOrMessage
				 if exportParams.exportImageFormat == "TIFF" then
					firstPhotoPath = LrPathUtils.replaceExtension(firstPhotoPath, "tiff")
					LrFileUtils.move(pathOrMessage, firstPhotoPath)
				 end

				command = command .. " \"" .. firstPhotoPath .. "\""
			end
			if  i >= 2 then
				if bracket == nil then
					bracket = string.len(firstPhotoPath) .. ":" .. firstPhotoPath
				end
				bracket = bracket .. string.len(pathOrMessage) .. ":" .. pathOrMessage
			end
		end
	end
	
	if command ~= nil then
	
		if bracket ~= nil then
			command = command .. " -MPHDRBracket " .. LrStringUtils.encodeBase64(bracket)
		end

		local resPath = nil
		local catalog = nil
		for photo in exportSession:photosToExport() do
			if resPath == nil then
				local photoPath = photo.path
				catalog = photo.catalog
				local pathName = LrPathUtils.removeExtension(photoPath)
				pathName = pathName .. "_" .. "MPExtrasPluginAppName" .. "-edit"
				resPath = LrPathUtils.addExtension(pathName, "tiff")
				local pathIndex = 0
				while LrFileUtils.exists(resPath) do
					local indexedPathName = pathName .. "_" ..  pathIndex
					resPath = LrPathUtils.addExtension(indexedPathName, "tiff")
					pathIndex = pathIndex + 1
				end
				break
			end
		end
		
		if resPath ~= nil then
			command = command .. " -MPResPath " .. LrStringUtils.encodeBase64(resPath)

		end

		if selectedPreset ~= nil then
			command = command .. " -MPPresetName " .. "\"" .. selectedPreset.name .. "\"" .. " -MPPresetID " .. "\"" .. selectedPreset.id .. "\""
		end
		local hideWindow = exportContext.propertyTable.hide
		if hideWindow == true and selectedPreset ~= nil then
			command = command .. " -MPHideWindow"
		end
		
		if numberOfRenderedPhotos > 1 then
			local alignment = exportContext.propertyTable.alignment
			if alignment then
				command = command .. " -MPAlignment"
			end
		
			local ghostsReduction = exportContext.propertyTable.ghostsReduction
			if ghostsReduction then
				command = command .. " -MPGhostsReduction"
			end
		end
		
		local chromaticAberrationReduction = exportContext.propertyTable.chromaticAberrationReduction
		if chromaticAberrationReduction then
			command = command .. " -MPChromaticAberrationReduction"
		end
		
		command = command .. " -MPWait" .. " -MPAppBundlePath \"" .. "MPAppBundlePathTemplateKeyword\""
		
		LrTasks.execute(command)
		
		if catalog ~= nil and resPath ~= nil and LrFileUtils.exists(resPath) then
			local addedPhoto = nil
			catalog:withWriteAccessDo("0",
				function(context)
					addedPhoto = catalog:addPhoto(resPath, nil, nil)

					if addedPhoto ~= nil then
						if collections ~= nil then
							for k, v in pairs(collections) do
								v:addPhotos({addedPhoto})
							end
						end
						
						local currentKeywords = addedPhoto:getRawMetadata("keywords")
						if (numberOfElementsInArray(currentKeywords) == 0) and (numberOfElementsInArray(keywords) > 0) then
							for k, v in pairs(keywords) do
								addedPhoto:addKeyword(v)
							end
						end

						-- restoring metadata
						addedPhoto:setRawMetadata("copyName", copyName)
						addedPhoto:setRawMetadata("title", title)
						addedPhoto:setRawMetadata("caption", caption)
						addedPhoto:setRawMetadata("copyright", copyright)
						addedPhoto:setRawMetadata("copyrightState", copyrightStatus)
						addedPhoto:setRawMetadata("creator", creator)
						addedPhoto:setRawMetadata("location", location)
						addedPhoto:setRawMetadata("rating", rating)
						addedPhoto:setRawMetadata("label", label)
					end
				end
			)

			if addedPhoto ~= nil then
				catalog:setSelectedPhotos(addedPhoto, {addedPhoto})
			end
		end
		

	end
	
end

local lPresetsLoaded = false

function ExportProcessRenderedPhotos.startDialog( propertyTable )
	lPresetsLoaded = false
	local startTime = LrDate.currentTime()
	while not g_MPExtrasPluginAppName_presetsLoaded do
		local currentTime = LrDate.currentTime()
		if currentTime - startTime > 5 then 
			break 
		end
	end

	if g_MPExtrasPluginAppName_presetsLoaded then
		if #g_MPExtrasPluginAppName_presets == #g_MPExtrasPluginAppName_presetGroups then
			lPresetsLoaded = true
			validPresets = {}
			validGroups = {}
			local validCount = 0
			for index, groupName in pairs(g_MPExtrasPluginAppName_presetGroups) do
				local groupPresets = g_MPExtrasPluginAppName_presets[index]
				if #groupPresets > 0 then
					validCount = validCount + 1
					validGroups[validCount] = groupName
					validPresets[validCount] = groupPresets
				end
			end
			local presets = validPresets
			local group = propertyTable.presetGroup
			local groupPresets = nil

			if group == nil then
				propertyTable.presetGroup = 1
				group = 1
			else 
				if type(group) ~= "number" then
					propertyTable.presetGroup = 1
					group = 1
				end
			end

			if group >= 2 then
				groupPresets = presets[group - 1]
			end
			if groupPresets ~= nil then
				if #groupPresets == 0 then
					propertyTable.presetGroup = 1
				end
			else
				propertyTable.presetGroup = 1
				propertyTable.hide = false
			end
		end
	else 
		LrDialogs.message("Could not load presets for MPExtrasAppName")
	end
end

function ExportProcessRenderedPhotos.updateExportSettings( exportSettings )
    exportSettings.exportImageFormat = exportSettings.LR_format
    exportSettings.LR_removeLocationMetadata = false
end

function ExportProcessRenderedPhotos.sectionsForTopOfDialog( viewFactory, propertyTable )
	if not lPresetsLoaded then
		return nil
	end
	local groupItems = {}
	local groups = validGroups
	local presets = validPresets

	local groupsCount = 0
	groupsCount = groupsCount + 1
	table.insert(groupItems, {title = LOC( "$$$/MPExtrasPluginAppName/Export/None=None" ), value = groupsCount})

	for index, groupName in pairs(groups) do 
		table.insert(groupItems, {title = groupName, value = groupsCount + 1})
		groupsCount = groupsCount + 1
	end
	
	
	propertyTable:addObserver( 'presetGroup', 
	function( properties, key, newValue )
		properties.preset = 1
		local presets = nil
		if newValue >= 2 then
			presets = validPresets[newValue - 1]
		end		
		if presets == nil then
			properties.hide = false
		end
	end )

	local result = nil
	
	local catalog = LrApplication.activeCatalog()
	local selectedPhotos = catalog:getTargetPhotos()
	local selectedPhotosCount = 0
	for index, photo in pairs(selectedPhotos) do
		selectedPhotosCount = selectedPhotosCount + 1
	end
	
	if groupsCount > 0 then
		if selectedPhotosCount > 1 then
			result = {
				{
					title = 'MPExtrasAppName',

					viewFactory:row {
						viewFactory: static_text {
							title = LOC( "$$$/MPExtrasPluginAppName/Export/Group=Group:" ),
						},
						viewFactory:popup_menu {
							value = LrView.bind('presetGroup'),
							items = groupItems,
							width_in_chars = 15,
						},
				
						viewFactory: static_text {
							title = LOC( "$$$/MPExtrasPluginAppName/Export/Preset=Preset:" ),
						},
						viewFactory:popup_menu {
							value = LrView.bind('preset'),
							items = LrView.bind(
							{
								key = 'presetGroup', 
								transform = function(value, fromTable)
									if not fromTable then
										return LrBinding.kUnsupportedDirection
									end
									local presetItems = {}
									local presets = validPresets
									local groupPresets = nil
									if value >= 2 then
										groupPresets = presets[value - 1]
									end
									if groupPresets ~= nil then
										for index, preset in pairs(groupPresets) do
											table.insert(presetItems, {title = preset.name, value = index})
										end
									else
										table.insert(presetItems, {title = "", value = 1})
									end
									return presetItems
								end
							}),
							enabled = LrView.bind(
							{
								key = 'presetGroup', 
								transform = function(value, fromTable)
									if not fromTable then
										return LrBinding.kUnsupportedDirection
									end
									local result = false
									if value >= 2 then
										result = validPresets[value - 1] ~= nil
									end
									return result
								end
							}),
							width_in_chars = 15,
						},
					}
				},
			}
		else
			result = {
				{
					title = 'MPExtrasAppName',

					viewFactory:row {
						viewFactory: static_text {
							title = LOC( "$$$/MPExtrasPluginAppName/Export/Group=Group:" ),
						},
						viewFactory:popup_menu {
							value = LrView.bind('presetGroup'),
							items = groupItems,
							width_in_chars = 15,
						},
				
						viewFactory: static_text {
							title = LOC( "$$$/MPExtrasPluginAppName/Export/Preset=Preset:" ),
						},
						viewFactory:popup_menu {
							value = LrView.bind('preset'),
							items = LrView.bind(
							{
								key = 'presetGroup', 
								transform = function(value, fromTable)
									if not fromTable then
										return LrBinding.kUnsupportedDirection
									end
									local presetItems = {}
									local presets = validPresets
									local groupPresets = nil
									if value >= 2 then
										groupPresets = presets[value - 1]
									end
									if groupPresets ~= nil then
										for index, preset in pairs(groupPresets) do
											table.insert(presetItems, {title = preset.name, value = index})
										end
									else
										table.insert(presetItems, {title = "", value = 1})
									end
									return presetItems
								end
							}),
							enabled = LrView.bind(
							{
								key = 'presetGroup', 
								transform = function(value, fromTable)
									if not fromTable then
										return LrBinding.kUnsupportedDirection
									end
									local result = false
									if value >= 2 then
										result = validPresets[value - 1] ~= nil
									end
									return result
								end
							}),
							width_in_chars = 15,
						},
					}
				}
			}
		end
	end
	return result
end

function intersectArrays(array1, array2)
	local result = {}
	local count = 0
	if array1 ~= nil and array2 ~= nil then
		for k1, v1 in pairs(array1) do
			for k2, v2 in pairs(array2) do
				if v1 == v2 then
					result[count] = v1
					count = count + 1
					break
				end
			end
		end
	end
	return result
end

function numberOfElementsInArray(array)
	local result = 0
	if array ~= nil then
		for k, v in pairs(array) do
			result = result + 1
		end
	end
	return result
end

-- For Debugging --

function writeToFileWithPrefix(table, file, prefix)
	file:write(prefix .. "{\n")
	for k, v in pairs(table) do
		file:write(prefix .. "\t[\"" .. tostring(k) .. "\"]=")
		local vType = type(v)
		if vType == "string" then
			file:write("\"".. v .. "\"\n")
		elseif vType == "number" then
			file:write(tostring(v) .. "\n")
		elseif vType == "boolean" then
			file:write(tostring(v) .. "\n")
		elseif vType == "table" then
			file:write("\n")
			local newPrefix = prefix .. "\t"
			writeToFileWithPrefix(v, file, newPrefix)
		else
			file:write("Unknown type: " .. vType .. "\n")
		end
	end
	file:write(prefix .. "}\n")
end

return ExportProcessRenderedPhotos
